@@ -141,8 +141,6 @@ sub _return_overrides ( $self, $req ) {
     return;
 }
 
-# TODO load only domains, used by app
-# TODO add overrides domains
 sub _return_locale ( $self, $req ) {
     state $locale_settings = {};
 
@@ -156,18 +154,25 @@ sub _return_locale ( $self, $req ) {
     }
 
     if ( !$self->{_cache}->{locale}->{$locale} ) {
+        my $app = $Pcore::Ext::APP->{ $self->{ext_app} };
 
         $locale_settings->{$locale} //= $ENV->{share}->read_cfg( 'data', "ext/locale/$locale.perl" );
 
-        # load locale domains
+        # load locale
         Pcore::Core::L10N::load_locale($locale) if !exists $Pcore::Core::L10N::MESSAGES->{$locale};
 
+        # get messages, used by app classes
+        my $locale_messages = $Pcore::Core::L10N::MESSAGES->{$locale};
+
+        # grep app messages, that have translation
+        my $messages = { $locale_messages->%{ grep { $locale_messages->{$_} } keys $app->{l10n}->%* } };
+
         my $plural_form_exp = $Pcore::Core::L10N::LOCALE_PLURAL_FORM->{$locale}->{exp} // 0;
 
         my $js = <<"JS";
             Ext.L10N.addLocale(
                 '$locale',
-                {   messages: @{[ to_json( $Pcore::Core::L10N::MESSAGES->{$locale}, canonical => 1 )->$* ]},
+                {   messages: @{[ to_json( $messages, canonical => 1 )->$* ]},
                     pluralFormExp: function (n) { return $plural_form_exp; },
                     settings: @{[ to_json( $locale_settings->{$locale}, canonical => 1 )->$* ]}
                 }
