@@ -1,4 +1,4 @@
-package Pcore::Ext v0.17.0;
+package Pcore::Ext v0.17.1;
 
 use Pcore -dist, -const;
 use Pcore::Ext::Base;
@@ -239,6 +239,9 @@ sub _build_apps ( $self, $tree ) {
             # add content
             $app->{content} .= "$class->{content}->$*;\n";
 
+            # add app l10n msgid
+            $app->{l10n}->@{ keys $class->{l10n}->%* } = ();
+
             # add api map
             for my $method_id ( sort keys $class->{api}->%* ) {
                 next if exists $added_methods->{$method_id};
@@ -248,9 +251,6 @@ sub _build_apps ( $self, $tree ) {
                 push $app->{api}->{ delete $class->{api}->{$method_id}->{action} }->@*, $class->{api}->{$method_id};
             }
 
-            # add locale domains
-            $app->{l10n_domain}->@{ keys $class->{l10n_domain}->%* } = ();
-
             return;
         };
 
