@@ -4,17 +4,12 @@ use Pcore -l10n;
 
 sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
     return {
-        listen => {
-            global => {
-                mask           => 'mask',
-                unmask         => 'unmask',
-                unmatchedRoute => 'onUnmatchedRoute',
-                remoteEvent    => 'onRemoteEvent',
-                login          => 'onLogin',
-                logout         => 'onLogout',
-                requestError   => 'onRequestError',
-                setLocale      => 'onSetLocale',
-            },
+        roles => [],
+
+        api => {
+            signin    => undef,    # $api->{'Auth/signin'},
+            signout   => undef,    # $api->{'Auth/signout'},
+            setLocale => undef,    # $api->{'Auth/set_locale'},
         },
 
         # FASHION THEME
@@ -22,10 +17,31 @@ sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
         accentColor => undef,
         darkMode    => \0,
 
+        defaultMask => {
+            transparent => \0,
+            html        => '<img src="/static/loader4.gif" width="100"/>',
+        },
+
+        listen => {
+            global => {
+                mask           => 'mask',
+                unmask         => 'unmask',
+                unmatchedRoute => 'onUnmatchedRoute',
+                remoteEvent    => 'onRemoteEvent',
+                requestError   => 'onRequestError',
+                signin         => 'onSignin',
+                signout        => 'onSignout',
+                setLocale      => 'onSetLocale',
+            },
+        },
+
         init => func [], <<"JS",
             var me = this;
 
-            me.mask();
+            // parse API methods
+            if (me.api.signin) me.api.signin = Ext.direct.Manager.parseMethod(me.api.signin);
+            if (me.api.signout) me.api.signout = Ext.direct.Manager.parseMethod(me.api.signout);
+            if (me.api.setLocale) me.api.setLocale = Ext.direct.Manager.parseMethod(me.api.setLocale);
 
             if (!!window.Fashion) {
                 var theme = {"dark-mode": me.darkMode ? "true" : "false"};
@@ -42,13 +58,25 @@ sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
 
             Ext.route.Router.suspend();
 
-            me.doLogin(null, null, function(res) {
+            me.initApp();
+
+            me.callParent(arguments);
+JS
+
+        initApp => func [], <<"JS",
+            var me = this;
+
+            me.mask();
+
+            me.doSignin(null, null, function(res) {
                 if (res.isSuccess()) {
 
-                    // store session data
-                    me.getViewModel().set('session', res.data || {});
+                    var session = me.checkSession(res.data);
 
-                    me.setLocale(res.data.locale, function () {
+                    // store session data
+                    me.getViewModel().set('session', session);
+
+                    me.setLocale(session.locale, function () {
                         Ext.route.Router.resume();
                     });
                 }
@@ -61,21 +89,42 @@ sub EXT_controller : Extend('Ext.app.ViewController') : Type('controller') {
                         html: $l10n->{'Error connecting to the server'} + '<br/>' + $l10n->{'Try again.'},
                         buttons: [{
                             text: 'retry',
-                            handler: 'onRetry'
+                            handler: function () {
+                                this.up('dialog').close();
+
+                                me.initApp();
+                            }
                         }]
                     }).show();
                 }
             });
+JS
 
-            me.callParent(arguments);
+        checkSession => func ['session'], <<'JS',
+
+            // compare roles
+            if (session.is_authenticated && !session.is_root) {
+                if (this.roles && this.roles.length) {
+                    session.is_authenticated = false;
+
+                    for (var role of this.roles) {
+                        if (session.permissions[role]) {
+                            session.is_authenticated = true;
+
+                            break;
+                        }
+                    }
+                }
+            }
+
+            if (!session.locale) session.locale = localStorage.locale || session.default_locale;
+
+            return session;
 JS
 
         # MASK
         mask => func [], <<'JS',
-            this.getView().setMasked({
-                transparent: true,
-                html: '<img src="/static/loader4.gif" width="100"/>',
-            });
+            this.getView().setMasked(this.defaultMask);
 JS
 
         unmask => func [], <<'JS',
@@ -91,6 +140,9 @@ JS
                 Ext.toast("Error: " + res.reason, 3000);
 JS
 
+        onRemoteEvent => func ['ev'], <<"JS",
+JS
+
         # TOKEN
         setToken => func [ 'token', 'persistent' ], <<'JS',
             this.removeToken();
@@ -119,7 +171,7 @@ JS
             if (Ext.L10N.getCurrentLocale() == locale) return;
 
             // store user locale in profile, if user is authenticated
-            if (this.getViewModel().get('session').is_authenticated) this.doUpdateLocale(locale);
+            if (this.getViewModel().get('session').is_authenticated) this.doSetLocale(locale);
 
             var me = this;
 
@@ -172,27 +224,36 @@ JS
             });
 JS
 
-        # LOGIN
-        onLogin => func [ 'username', 'password', 'persistent' ], <<"JS",
+        # SIGNIN
+        onSignin => func [ 'username', 'password', 'persistent' ], <<"JS",
             var me = this;
 
             me.mask();
 
-            me.doLogin(username, password, function (res) {
+            me.doSignin(username, password, function (res) {
                 me.unmask();
 
                 if (res.isSuccess()) {
 
+                    var session = me.checkSession(res.data);
+
+                    // not authenticated
+                    if (!session.is_authenticated) {
+                        Ext.toast($l10n->{'You have no permissions to access this area'}, 3000);
+
+                        return;
+                    };
+
                     // store API token
-                    me.setToken(res.data.token, persistent);
+                    me.setToken(session.token, persistent);
 
                     // set token and disconnect
-                    APP.getApplication().api.auth(res.data.token);
+                    APP.getApplication().api.auth(session.token);
 
                     // store session data
-                    me.getViewModel().set('session', res.data);
+                    me.getViewModel().set('session', session);
 
-                    me.setLocale(res.data.locale, function () {
+                    me.setLocale(session.locale, function () {
                         me.clearInterface();
 
                         me.redirectTo(Ext.util.History.getToken(), {force: true});
@@ -204,8 +265,8 @@ JS
             });
 JS
 
-        onLogout => func [], <<"JS",
-            this.doLogout();
+        onSignout => func [], <<"JS",
+            this.doSignout();
 
             // drop API token
             this.removeToken();
@@ -221,6 +282,22 @@ JS
             // redirect
             this.redirectTo(Ext.util.History.getToken(), {force: true});
 JS
+
+        # API
+        doSignin => func [ 'username', 'password', 'cb' ], <<"JS",
+            this.api.signin({
+                username: username,
+                password: password
+            }, cb);
+JS
+
+        doSignout => func [], <<"JS",
+            this.api.signout();
+JS
+
+        doSetLocale => func ['locale'], <<"JS",
+            this.api.setLocale(locale);
+JS
     };
 }
 
